#   Copyright 2020 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def predict_modelvis_component(img_dataset, vis_dataset, component_parms, storage_parms):
    """
    .. todo::
        This function is not yet implemented
        
    Predict model visibilities from a component list by analytical evaluation
    
    Apply PB models to the components prior to evaluation.
    
    Save the model visibilities in arr_name (default = 'MODEL')
    
    Optionally overwrite the model or add to existing model (incremental=T)

    """
